@@ -139,10 +139,21 @@ class Dispatch(object):
 			))
 		if self.username: pack['user'] = self.username + "@"
 		
+		packUpdate = {}
 		if isinstance(pack['sender'], tuple):
-			pack.update(dict(addrfrom='"%s" %s' % pack['sender'], smtpfrom=pack['sender'][1]))
+			packUpdate['addrfrom'] = '"%s" %s' % pack['sender']
+			packUpdate['smtpfrom'] = pack['sender'][1]
 		else:
-			pack.update(dict(addrfrom='- %s' % pack['sender'], smtpfrom=pack['sender']))
+			packUpdate['addrfrom'] = '- %s' % pack['sender']
+			packUpdate['smtpfrom'] = pack['sender']
+		
+		if pack['smtpfrom']:
+			if isinstance(pack['smtpfrom'], tuple):
+				packUpdate['smtpfrom'] = pack['smtpfrom'][1]
+			else:
+				packUpdate['smtpfrom'] = pack['smtpfrom']
+		
+		pack.update(packUpdate)
 		
 		if len(pack['to']) > 1:
 			pack.update(dict(addrto="- (%d)" % len(pack['to'])))
