@@ -22,7 +22,7 @@ class Pool(Thread):
 	method.
 	"""
 	
-	def __init__(self, interval=10, threads=4, jobs=10, timeout=60, **kw):
+	def __init__(self, interval=10, threads=4, jobs=10, timeout=60, polling=False, **kw):
 		"""Initialize the threadpool.
 		
 		@param interval: A delay, in seconds, between spawn runs.
@@ -38,6 +38,11 @@ class Pool(Thread):
 		@param timeout: The amount of time, in seconds, a thread is
 		                allowed to sit idle before dying of starvation.
 		@type timeout: int
+		
+		@param polling: Enable or disable the periodic polling
+		                mechanism.  Disabled, threads will be created,
+		                as required, when work is enqueued.
+		@type polling: bool
 		"""
 		
 		super(Pool, self).__init__()
@@ -49,6 +54,7 @@ class Pool(Thread):
 		self._threads = threads
 		self._jobs = jobs
 		self._timeout = timeout
+		self._polling = False
 		self._kw = kw
 		
 		log.debug("Thread pool created.")
@@ -56,9 +62,6 @@ class Pool(Thread):
 	def enqueue(self, work):
 		"""Enqueue a Message instance.
 		
-		@todo: Optionally, this method should immediately trigger the
-		       management thread, allowing for almost immediate latency.
-		
 		@param work: The unit of work can be any callable that returns a
 		             three-item tuple containing the sender and
 		             recipient addresses and a properly formatted MIME
@@ -66,9 +69,22 @@ class Pool(Thread):
 		             instance of the Message class or subclass.
 		@type work: callable
 		"""
+
+		if callable(work):
+			self._queue.put(work())
+		else:
+			self._queue.put(work)
+		
+		optimum_threads = min(self._threads, math.ceil(self._queue.qsize() / float(self._jobs)))
+
+		if not self._polling and not self._queue.empty() and self._pool < optimum_threads:
+			log.debug("Work enqueued.  Spawning %d threads." % (optimum_threads - self._pool))
+			for i in range(int(optimum_threads - self._pool)):
+				self.spawn()
+
+		else:
+			log.debug("Work enqueued.")
 		
-		log.debug("Work enqueued.")
-		self._queue.put(work)
 	
 	def shutdown(self):
 		"""Quit the management thread and shutdown the queue."""
@@ -101,7 +117,7 @@ class Pool(Thread):
 			
 			if not self._queue.empty() and self._pool < optimum_threads:
 				log.debug("Creating %d threads." % (optimum_threads - self._pool))
-				for i in xrange(int(optimum_threads - self._pool)):
+				for i in range(int(optimum_threads - self._pool)):
 					self.spawn()
 					
 			self._finished.wait(self._interval)
@@ -143,6 +159,8 @@ class MailPool(Pool):
 			)
 		message.plain = "Hello world!"
 		pool.enqueue(message)
+		# wait for message to send
+		pool.shutdown()
 
 	"""
 	
